@@ -434,6 +434,17 @@ class HealpixBase:
 
             return hmap.nest2uniq(self.nside, pix)
 
+    @property    
+    def uniq(self):
+        """
+        Get an array with the NUNIQ numbers for all pixels
+        """
+
+        if self.is_moc:
+            return self._uniq
+        else:
+            return self.pix2uniq(range(self.npix))
+        
     def nest2pix(self, pix):
         """
         Get the corresponding pixel in the current grid for a pixel in NESTED
