@@ -53,9 +53,7 @@ class HealpixMap(HealpixBase):
 
     The maps are array-like, that is, the can be casted into a regular numpy 
     array (as used by healpy), are iterable (over the pixel values) and can be
-    used with built-in function such as ``sum`` and ``max``. For MOC maps, if
-    you want to keep the `UNIQ` pixel numbers in a ``recarray``, you have to use
-    ``data()``.
+    used with built-in function such as ``sum`` and ``max``.
 
     You can also access the value of pixels using regular numpy indexing 
     with ``[]``. For MOC maps, no specific pixel ordering is guaranted. For a 
@@ -63,10 +61,10 @@ class HealpixMap(HealpixBase):
     corresponding UNIQ pixel number using ``m.pix2uniq(ipix)``.
 
     Args:
-        data (array): ``HealpixMap``, for an emty map, or value for an 
-            initialize map. A MOC map is recarray whose first field is called 
-            'UNIQ'. The second field, if it exists, will initialize the contents,
-            otherwise the map will be zero-initialized.
+        data (array): Values to initialize map. Zero-initialized it not provided.
+            The map NSIDE is deduced from the array size, unless ``uniq`` 
+            is specified in which case this is considered a multi-resolution map.
+        uniq (array): List of NUNIQ pixel number to initialize a MOC map. 
         order (int): Order of HEALPix map.
         nside (int): Alternatively, you can specify the NSIDE parameter.
         scheme (str): Healpix scheme. Either 'RING', 'NESTED' or 'NUNIQ'
@@ -79,6 +77,7 @@ class HealpixMap(HealpixBase):
 
     def __init__(self,
                  data = None,
+                 uniq = None,
                  order = None,
                  nside = None,
                  scheme = 'ring',
@@ -91,41 +90,33 @@ class HealpixMap(HealpixBase):
         if data is not None:
             # Initializes map contents
 
-            # Standarize to array
-            data = array(data)
+            # Initialize base
+            if uniq is not None:
+                #MOC
+                
+                super().__init__(uniq = uniq)
 
-            # Single or multi resolution?
-            if (data.dtype.names is not None and
-                data.dtype.names[0] == "UNIQ"):
-                # Interpret as a multi-order coverage (MOC) map
-
-                data = data.view(np.recarray)
-
-                super().__init__(uniq = data.field(0))
-
-                if len(data.dtype.names) > 1:
-                    self._data = data.field(1)
-                else:
-                    # Empty map
-                    self._data = np.zeros(self.npix, dtype = dtype)
+                if len(data) != self.npix:
+                    raise ValueError("Data size mismatch.")
                 
             else:
-                # Single resolution map
+                # Single resolution. From array size itself
+                
+                nside = hp.npix2nside(len(data))
 
-                order = hp.nside2order(hp.npix2nside(data.size))
-
-                super().__init__(order = order,
+                super().__init__(nside = nside,
                                  scheme = scheme)
 
-                self._data = data
+            # Set data
+            self._data = array(data)
 
         else:
             # Empty map
             
-            super().__init__(nside = nside,
-                             order = order,
+            super().__init__(order = order,
+                             nside = nside,
                              scheme = scheme)
-
+        
             self._data = np.zeros(self.npix, dtype=dtype)
 
         # Other properties
@@ -165,10 +156,7 @@ class HealpixMap(HealpixBase):
                 uniq = hdu.data.field(uniq_field).ravel()
                 contents = hdu.data.field(field).ravel()
 
-                data = np.rec.fromarrays([uniq, contents],
-                                 names = ['UNIQ', 'CONTENTS'])
-
-                m = cls(data, density = density)
+                m = cls(contents, uniq, density = density)
 
             else:
                 # Sigle resolution
@@ -394,10 +382,7 @@ class HealpixMap(HealpixBase):
                     # Clear buffer at this order
                     nbuffer[step] = 0
 
-        map_data = np.rec.fromarrays([map_uniq, map_data],
-                                     names = ['UNIQ', 'CONTENTS'])
-                    
-        return cls(map_data, density = density)
+        return cls(map_data, map_uniq, density = density)
 
     @classmethod
     def adaptive_moc_mesh(cls, max_nside, split_fun, density = False,
@@ -422,10 +407,9 @@ class HealpixMap(HealpixBase):
         
         base = HealpixBase.adaptive_moc_mesh(max_nside, split_fun)
 
-        data = np.rec.fromarrays([base._uniq, np.zeros(base.npix, dtype = dtype)],
-                                 names = ["UNIQ", "CONTENTS"])
-
-        return cls(data, density = density)
+        return cls(np.zeros(base.npix, dtype = dtype),
+                   base._uniq,
+                   density = density)
         
     @classmethod
     def moc_from_pixels(cls, nside, pixels, nest = False, density = False,
@@ -447,10 +431,9 @@ class HealpixMap(HealpixBase):
 
         base = HealpixBase.moc_from_pixels(nside, pixels, nest = nest)
 
-        data = np.rec.fromarrays([base._uniq, np.zeros(base.npix, dtype = dtype)],
-                                 names = ["UNIQ", "CONTENTS"])
-
-        return cls(data, density = density)
+        return cls(np.zeros(base.npix, dtype = dtype),
+                   base._uniq,
+                   density = density)
 
     @classmethod
     def moc_histogram(cls, nside, samples, max_value, nest = False, weights = None):
@@ -648,29 +631,13 @@ class HealpixMap(HealpixBase):
 
         return self._density
 
-    
-    def data(self, contents_label = 'CONTENTS', uniq_label = 'UNIQ'):
+    @property
+    def data(self):
         """
         Get the raw data in the form of an array.
-
-        Args:
-            contents_label (str): Name of the contents column (2nd), 
-                for multi-resolution maps.
-            uniq_label (str): Name of the UNIQ pixel numbers column (1st),
-                for multi-resolution maps.
-
-        Return:
-            array or recarray: For single and multi-resolution maps, respectively.
         """
 
-        if self.is_moc:
-
-            return np.rec.fromarrays([self._uniq, self._data],
-                                     names = [uniq_label, contents_label])
-            
-        else:
-
-            return self._data
+        return self._data
         
     @property
     def dtype(self):
@@ -925,10 +892,9 @@ class HealpixMap(HealpixBase):
                 new_density = map0._density and map1._density
 
                 # Create new map
-                new_data = np.rec.fromarrays([new_uniq[:pix_new], new_data[:pix_new]],
-                                              names = ["UNIQ", "CONTENTS"])
-
-                return HealpixMap(new_data, density = new_density)
+                return HealpixMap(new_data[:pix_new],
+                                  new_uniq[:pix_new],
+                                  density = new_density)
                 
             else:
 
@@ -1244,28 +1210,21 @@ class HealpixMap(HealpixBase):
         Return:
             scalar or array
         """
-        
-        if self.is_moc:
 
-            pixels,weights = self.get_interp_weights(theta, phi)
+        pixels,weights = self.get_interp_weights(theta, phi)
 
-            values = self[pixels]
+        values = self[pixels]
             
-            if not self._density:
-                # Split the pixels up to corresponding maximum order
-                
-                nside = hmap.uniq2nside(self._uniq[pixels])
-
-                npix_ratio =  int(4**self.order) / nside / nside
-                
-                values = values / npix_ratio
-
-            return sum(values * weights)
+        if self.is_moc and not self._density:
+            # Split the pixels up to corresponding maximum order
             
-        else:
-
-            return hp.get_interp_val(self._data, theta, phi, nest = self.is_nested)
+            nside = hmap.uniq2nside(self._uniq[pixels])
+            
+            npix_ratio =  int(4**self.order) / nside / nside
+            
+            values = values / npix_ratio
 
+        return np.matmul(weights, values)
 
     def moc_sort(self):
         """
